
//    .----------------------------------------------------------------------------------------------------------------------------------------------------------------.
//    |                                                     ComCan - CanSat-Team 2016 des Ökumenischen Gymnasiums                                                      |
//    '----------------------------------------------------------------------------------------------------------------------------------------------------------------'
//     von Louis Fahrenholz, Adrian Lipow, Julien Peteaux, Sandra Vöth (CanSat-Team "ComCon", Bremen) und Daniel Wendt-Fröhlich, DL2AB (Hackerspace Bremen e.V.)
//
//     CC-BY-SA 3.0 - Mai-Sep 2016, Bremen(GER)
//     "Dieses Werk ist unter einer Creative Commons Lizenz vom Typ Namensnennung - Weitergabe unter gleichen Bedingungen 3.0 Deutschland zugänglich. 
//     Um eine Kopie dieser Lizenz einzusehen, konsultieren Sie http://creativecommons.org/licenses/by-sa/3.0/de/ oder wenden Sie sich brieflich an 
//     Creative Commons, Postfach 1866, Mountain View, California, 94042, USA."
//
//     Base64-Code "binToAscii" (Public Domain) von spex@neptune.sinister.com
//
//    ------------------------------------------------------------------------------------------------------------------------------------------------------------------
//        
//     Die Hardware ist das "qbcan v1.1"-Set:
//                                  - Arduino ProMicro *
//                                  - I2C-Temperatur-/Drucksensor-Platine 'BMP180'
//                                  - Funkmodul 'RFM69' *
//                                      * ebenfalls als Module für das Bodenstations-Modem verwendet
//     Zusätzlich werden im CanSat verwendet: 
//                                  - ein generisches GPS-Modul mit NMEA-Ausgabe (RX von Hardware-Serial1, 9600bd), 
//                                  - serielle Kamera 'VC0706' (RX=Pin8 TX=Pin9, 9600bd) 
//                                  - SD-Datalogger (eigener ArduinoNano an TX von Hardware-Serial1, 9600bd).
//
//     Alle Daten der Sensoren werden in ein 50byte großes Paket verpackt, dass gesendet und per Datalogger in einem LogFile gespeichert wird.
//     Das schließt ebenfalls aufgenommene Fotos ein, die ein Base64-Encoding und html/img-Wrapping durchlaufen haben.
//     Es gibt mehrere Betriebsmodi. 
//                 0 - regelmäßiges Senden von GPS-Daten, Temperatur und Druck (default)
//                 1 - Foto 160x120px aufnehmen und senden *
//                 2 - Foto 640x480px aufnehmen und nur speichern *
//                 3 - ein einfacher Ping *
//                     * nach Ablauf wird wieder der Modus 0 eingestellt
//
//    ------------------------------------------------------------------------------------------------------------------------------------------------------------------
//
//                                                                        Dosensatellit  >()_)
//
//    .----------------------------------------------------------------------------------------------------------------------------------------------------------------.
//    |                                                          UMGEBUNGSPARAMETER UND FUNKTIONEN                                                                     |
//    '----------------------------------------------------------------------------------------------------------------------------------------------------------------'
//                                                 .------------------------------------.
//                                                 |            BIBLIOTHEKEN            |
//                                                 '------------------------------------'
#include <qbcan.h>                        // QBCAN: Funkmodul, Temperatur-Sensor, Drucksensor
#include <Wire.h>                         // I2C-Bus für Sensoren
#include <TinyGPS++.h>                    // GPS-Modul
#include <Adafruit_VC0706.h>              // Kamera
#include <SoftwareSerial.h>               // Serielle Verbindung zur Kamera


// Betriebsmodi
byte CanSatMode = 0;                       // Betriebsmodus ( 0 = GPS/Temp/Druck-Intervall - 1 = kleines Foto senden - 2 = großen Foto speichern - 3 = Ping)
bool debug = 1;                            // schaltet serielle Ausgaben ein


//                                                 .------------------------------------.
//                                                 |          PAYLOAD-BEFEHLE           |
//                                                 '------------------------------------'
char payload[50];                           // Die zu sendende 50Byte-Payload (Char-Array! Größe nicht ändern!) 

void clearPayload()                         // überschreibt die payload mit Leerzeichen
{
  byte i = 0;
  while (i <= 48){                          // Für die Zeichen 0 - 48:
    // payload[i] = 20;                        // 20 = Leerzeichen
    payload[i] = ' ';
    i++;
  }
  payload[49] = 0;                          // die abschließende 0 am Zeichenstring-Ende(49)
}


//                                                 .------------------------------------.
//                                                 |          GPS-VERARBEITUNG          |
//                                                 '------------------------------------'
//                                ließt kontinuierlich die eingehenden GPS-Daten und wertet diese aus.
//                                Die für uns relevanten Daten werden zu einem Ausgabestring verkettet:
//                                Anzahl der Satelliten; Uhrzeit; Breitengrad; Längengrad; Höhe;
//                                Das GPS-Modul ist am HardwareSerial ("Serial1") angeschlossen.

String gpsOut;                              // Arbeitsstring
char gpsString[40];                         // GPS-Ausgabe - genauso breit wie payload

TinyGPSPlus gps;  // TinyGPS++ object       // Objekt zur Verarbeitung der kontinuierlich eingehenden GPS-Daten


void gpsData()                              // Liest die angegebenen Zeichen aus den NMEA-Stream und aktualisiert die Werte entsprechend im GPS-String (Teil der Payload)
{
    gpsOut = gps.satellites.value(); 
    gpsOut+= ";"; 
    if (gps.time.hour() < 10) gpsOut+='0';  // Diese Zeilen halten einstellige Werte für Stunde, Minute, Sekunden zweistellig
    gpsOut+=gps.time.hour();
    gpsOut+= ":";  
    if (gps.time.minute() < 10) gpsOut+='0';
    gpsOut+=gps.time.minute(); 
    gpsOut+= ":"; 
    if (gps.time.second() < 10) gpsOut+='0';
    gpsOut+=gps.time.second(); 
    gpsOut+=";";
    gpsOut+=String(gps.location.lat(),8); 
    gpsOut+=";";
    gpsOut+=String(gps.location.lng(),8);
    gpsOut+=";"; 
    gpsOut+=gps.altitude.meters();
    gpsOut.toCharArray(gpsString,40);        // übergibt den verketteten String an einen größenbeschränkten String, damit immer Platz für die Sensordaten bleibt.
}


//                                                 .------------------------------------.
//                                                 |  BMP(Temp+Druck) + GPS AN PAYLOAD  |
//                                                 '------------------------------------'
//                                ließt die Sensoren aus und verkettet die Daten mit der GPS-Ausgabe zu einer payload

BMP180 bmp;                                 // Objekt für die Sensor-Verarbeitung
double Temperatur,Pressure ;                // Variablen für Temperatur und Druck
char senstemp[9];

void getdata()
{
  bmp.getData(Temperatur,Pressure);         // ließt die Daten aus den Sensoren aus (per I2C, Objekt aus qbcan)
  sprintf(senstemp,";%dC;%d",(int)Temperatur,(int)Pressure);  // verkettet die Sensoren-Werte zu einem lesbaren String
  byte i = 0;
  while (i <= 40){                          // schreibt den GPS-String (40Byte) in die Payload
    if (gpsString[i] > 0) payload[i] = gpsString[i];
    i++;
  }
  i = 0;
  while (i <= 9){                           // schreibt die Sensordaten in die Payload
    payload[40+i] = senstemp[i];
    i++;
  }
  payload[49] = 0;                          // schreibt die obligatorische 0 an den Ende des Strings
}

//                                                 .------------------------------------.
//                                                 |       FUNKMODUL + DATALOGGER       |
//                                                 '------------------------------------'
//                                sendet die Payload mit unten angegebenen Parametern.
//                                RFM69-Code ist von qbcan - es fehlen noch weitere Angaben zur Modulation und Betriebsart.
//                                Der Datalogger wird lediglich an den TX-Pin vom Hardware-Serial (Serial1) angeschlossen und läuft autark.
//                                Die Payload wird dafür nur seriell ausgegeben.



// https://lowpowerlab.com/forum/moteino/rfm69hw-driverange-test/msg288/?PHPSESSID=a0d72b66bd05470a39ca53a6ad29d940

RFM69 radio;
#define NODEID        2                   // Node-ID ist einzigartig für jedes Gerät (der CanSat ist Node 2, die Bodenstation ist Node 1)
#define NETWORKID     166                 // Netzwerk-ID muss bei allen gleich sein
#define GATEWAYID     1                   // Angabe des Empfänger-Nodes 
#define ENCRYPTKEY    "OeGymTComConHSHB"  // Verschlüsselungspasswort muss exakt 16 Zeichen lang sein
bool promiscuousMode = false;             // ließt alle Pakete im Netzwerk wenn 'true', ließt nur an diesen Node adressierte Pakete wenn 'false'
char received[5];                         // String für empfangene Pakete

void senddata()                           // sendet die payload...
{
  radio.send(GATEWAYID, payload, 50);     // ...an das Funkmodul - GATEWAYID ist der Empfänger (=Node 1)
  Serial1.println(payload);               // ...an den Datalogger an TX vom (Hardware-)Serial1
  if (debug) Serial.println(payload);     // ...an die Debug-Ausgabe per USB
}


void receiveCommand()                     // liest empfangene Bytes vom Funkmodul und stellt den entsprechenden CanSat-Mode ein
{
  for (byte i = 0; ( (i < radio.DATALEN) || (i <= 5) ) ; i++){   // verpackt empfangene Bytes in den 5Byte großen String 'received'
    received[i]=(char)radio.DATA[i];
  }
  if (received[1] == '0') CanSatMode = 0; // stellt den CanSate-Mode entsprechend des zweiten Bytes(Pos.1) ein. 
  if (received[1] == '1') CanSatMode = 1; // Ggf. könnte man hier eine Fehlerkorrektur einfügen, das RFM69 hat diese jedoch Hardwareseitig eingebaut.
  if (received[1] == '2') CanSatMode = 2;
  if (received[1] == '3') CanSatMode = 3;  
}

//                                                 .------------------------------------.
//                                                 |             Intervall              |
//                                                 '------------------------------------'
//                                Hier landen alle Befehle, die in einer dauerhaften Schleife ausgeführt werden müssen, auch während der Bildverarbeitung.
//                                -Setzt den Watchdog-Timer zurück
//                                -Eingehende GPS-Daten müssen dauerhaft eingelesen und verarbeitet werden.
//                                -Auch darf das Sende-Intervall nicht unterbrochen werden. Ggf. werden die Sensordaten zwischen den Zeilen der Bildübertragung eingeschoben.
//                                 Diese lassen sich im LogFile händisch oder mit dem Kommandozeilen 'sed' herausfiltern und auseinander splitten.
int intervallAbstand=2000;                 // Abfrage-Intervall in ms.
unsigned long intervallMillis;             // temporäre Ablage für die Arduino-Zeit

void intervall() {
  
    // Liest kontinuierlich eingehende GPS-Daten, wenn diese eingegangen sind und führt die GPS-Verarbeitung aus.
  while (Serial1.available() > 0) {
    if (gps.encode(Serial1.read())) {
      gpsData();      
    }
  }

    // wird ausgeführt, wenn die vorgegebene Intervallzeit abgelaufen ist, Ablauf ist selbsterklärend
  if (millis() >= intervallMillis) {
    getdata();
    senddata();
    clearPayload();
    intervallMillis = millis()+intervallAbstand;
    
  } 
}

//                                                 .------------------------------------.
//                                                 |        BASE64 - ENCODER            |
//                                                 '------------------------------------'
//                                Base64-Encoding macht aus je 3 Bytes einer binären Datei 4 Bytes ASCII-Text. Dieses wird z.B. auch für Dateianhänge in E-Mails benutzt.
//                                Wir verwenden dieses Encoding zum Versenden der JPEG-Dateien vom Kameramodul.
//                                Dadurch kann das Foto über serielle Terminals verschickt und auch direkt in einer html-Datei eingebunden werden.
//                                Durch diese Ausgabe ist es auch möglich, die Bilder direkt angezeigt zu bekommen, wenn man das Logfile im Browser öffnet.
//                                Beim Encoding werden die 24bit von 3 Bytes auf 4x 6bit-Zahlenwerde verschoben. Jedem dieser 64 möglichen Werte ist ein ASCII-Zeichen zugewiesen.
//                                Am Ende übrig bleibende Bytes (weniger als 3) am Ende eines Strings müssen in der Ausgabe mit je einem '=' aufgefüllt werden.

void binToAscii(char * outstring, uint8_t byte0, uint8_t byte1, uint8_t byte2, uint16_t count)
{
  //outstring points to 4char string
  // byteX are the three possible input bytes
  // count marks the number of input bytes, this should 
  //       always be 0 unless it is the last one or two input bytes
  uint8_t temp1=0, temp2=0, temp0=0;      // temporäre Bytes für die Verarbeitung
  char outA=0, outB=0, outC=0, outD=0;    // einzelne Bytes für die Ausgabe
  char base64chars[]= "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";  // Jedes dieser Zeichen entspricht in der Position einer der 64(=6Bit) Möglichkeiten
 
  // pad properly to account for the number of input pieces   //verschiebt die Bytes der Engabe entsprechend der Leerbyte-Angabe "count" in die richtige Arbeitsvariable
  if (count == 0){
  temp0 = byte0;
  temp1 = byte1;
  temp2 = byte2;
  }
  if (count == 1){
  temp0 = byte2;
  temp1 = 0;
  temp2 = 0;
  }
   if (count == 2){
  temp0 = byte1;
  temp1 = byte2;
  temp2 = 0;
  }

  // For some reason, you can't combine these next two blocks
  
  // bit math to break out 4 separate 6 bit pieces from the 3 8 bit inputs
  // and use them to index the base64 character set array
  outA=base64chars[temp0 >> 2];                              // unterteilt die 24bits der 3-Eingabebytes in 4 Bytes durch Bitverschiebung und Addition   
  outB=base64chars[((temp0 & 0x03) << 4) + ( temp1 >> 4)];
  outC=base64chars[((temp1 & 0x0f) << 2) + ((temp2 & 0xc0) >> 6)];
  outD=base64chars[(temp2 & 0x3f)]; 

  //load the chars into the output string
  outstring[0]=outA;                                         // schiebt die errechnet Ausgabewerte in einen zusammenhängenden Ausgabestring
  outstring[1]=outB;
  outstring[2]=outC;
  outstring[3]=outD;
  
  // add padding                                             // Sind (meist am Ende der Ausgabe) weniger als 3 Bytes übrig, wird für jedes leere Ausgabebyte ein '=' eingesetzt
  if (count == 1){
    outstring[2]='=';
    outstring[3]='=';
  }
  if (count == 2){
    outstring[3]='=';
  }
}

//                                                 .------------------------------------.
//                                                 |     BILD AUFNEHMEN UND SENDEN      |
//                                                 '------------------------------------'
//                                Base64-Encoding macht aus je 3 Bytes einer binären Datei 4 Bytes ASCII-Text. Dieses wird z.B. auch für Dateianhänge in E-Mails benutzt.
//                                Wir verwenden dieses Encoding zum Versenden der JPEG-Dateien vom Kameramodul.
//                                Dadurch kann das Foto über serielle Terminals verschickt und auch direkt in einer html-Datei eingebunden werden.
//                                Durch diese Ausgabe ist es auch möglich, die Bilder direkt angezeigt zu bekommen, wenn man das Logfile im Browser öffnet.
//                                Beim Encoding werden die 24bit von 3 Bytes auf 4x 6bit-Zahlenwerde verschoben. Jedem dieser 64 möglichen Werte ist ein ASCII-Zeichen zugewiesen.
//                                Am Ende übrig bleibende Bytes (weniger als 3) am Ende eines Strings müssen in der Ausgabe mit je einem '=' aufgefüllt werden.



SoftwareSerial cameraconnection = SoftwareSerial(8, 9);          // Serielle Schnittstelle für Kamera
Adafruit_VC0706 cam = Adafruit_VC0706(&cameraconnection); 

void Picture() {                                                 // Routine für die Kamera-Aufnahme - je nach CanSatMode: Auflösung, Senden+Speichern oder nur Speichern

     // Auflösung einstellen. 
     // Laut Manual zum Kameramodul ist danach ein Reset (und Re-Initialisieren) notwendig: 
     // " When changing the size，it needs to be reset or reconnect power, once it has been changed,the parameters will remain even disconnect power. "
     // Wenn beides (reset und begin) nicht gemacht wird, führt das zu massiven Problemen bis hin zum Aufhängen des AVRs...
     
  cam.reset();                                                   // Kamera-Reset, löscht ggf. noch vorhandene Bilder 
  if (debug) {
    char *reply = cam.getVersion();
    Serial.println(reply);
  }

  if (CanSatMode == 1) cam.setImageSize(VC0706_160x120);         // kleine Auflösung bei Modus 1 (Bild senden)
  if (CanSatMode == 2) cam.setImageSize(VC0706_640x480);         // größte Auflösung bei Modus 2 (Bild speichern)
  //cam.setImageSize(VC0706_320x240);                            // mittlere Auflösung
  cam.reset();                                                   // Nach Änderung der Auflösung ist ein Reset notwendig, siehe Manual zu VC0706

  
  cam.setMotionDetect(false);                                    // schaltet die Bewegungserkennung und selbstständige Auslösung zweifelsfrei aus
  byte imgsize = cam.getImageSize();                             // gibt die Auflösung seriell aus
  if ( (imgsize == VC0706_640x480) && debug ) Serial.println("640x480");
  if ( (imgsize == VC0706_320x240) && debug ) Serial.println("320x240");
  if ( (imgsize == VC0706_160x120) && debug ) Serial.println("160x120");
  
  cam.takePicture();                                              // löst die Bildaufnahme aus
  
  if (! cam.takePicture()) {                                      // Fehlermeldung falls Bildaufnahme fehlgeschlagen ist
    if (debug) Serial.println("Failed to snap!");
    radio.send(GATEWAYID, "fail", 4);
  } else {
    if (debug) Serial.println("Picture taken!");
  }
  uint16_t jpglen = cam.frameLength();                            // ließt die Bildgröße aus und hinterlegt diese für die weitere Verarbeitung
  if (debug) {
    Serial.print(jpglen, DEC);
    Serial.println(" byte image.");
  }
  delay(300);

  uint16_t imageByteCount=0;                                       // Zählt wieviel Bytes gelesen wurden (für die 'Restprüfung' im Base64-Encoding)
  uint8_t old, older, oldest, oldcount;                            // Base64-Eingabe-Puffer (3Bytes, oldcount nur wenn weniger Bytes übrig bleiben)
  char base64Text[]="****";                                        // Base64-Ausgabe-Puffer (4Bytes)
  int bufferPos;                                                   // Position im Kameraspeicher
  byte ploadPos = 0;                                               // Position in der Payload
  clearPayload();                                                  // Payload leeren
  byte k; 
  char htmlIn[]="<p><img src=\'data:image/jpeg;base64,";           // String vor der base64-Ausgabe, der die Darstellung im Browser erleichtert
  char htmlOut[]="\' />";                                          // String nach der base64-Ausgabe,  "
  for (k = 0; k < sizeof(htmlIn) - 1; k++){                        // fügt die Preamble in die Payload ein
    payload[ploadPos] = htmlIn[k];
    ploadPos++;
  }
  while (jpglen != 0) {                                            // Schleife, die den Kameraspeicher ausließt

    // read 64 bytes at a time;
    uint8_t *buffer;                                               // Zwischenspeicher der Kameradaten
    uint8_t bytesToRead = min(32, jpglen);                         // Paketgröße, die auszulesen ist (32 oder kleiner, wenn Speicher beinahe ausgelesen ist)
    buffer = cam.readPicture(bytesToRead);                         // Kamera auslesen
    
    for(bufferPos=0; bufferPos < bytesToRead; bufferPos++) {       // Vorbereitung für base64-Encoding
      imageByteCount++;                                            // zählt pro gelesenes Byte weiter
      oldest=older;                                                // alte Werte werden um jeweils eine Variable verschoben
      older=old;
      old=buffer[bufferPos];                                       // neues Byte wird aus dem Kamerapaket eingelesen
      oldcount=imageByteCount % 3;                                 // Modulo für Restbytes (wenn Paket kleiner als 3)
      
      if((oldcount)==0) {                                          // Wenn keine Restbytes vorhanden sind ...
        binToAscii(base64Text, oldest, older, old, oldcount);      //   ...Werte an Base64-Encoder übergeben...
        for (k=0; k <= 3; k++) {                                   //   ...und Ergebnis in die Payload eintragen
          payload[ploadPos] = base64Text[k];
          ploadPos++;
        }
      }
      if(ploadPos >= 48) {                                         //   Wenn Payload gefüllt ist...
        payload[ploadPos] = 0;                                     //     letztes Payloadzeichen ist eine '0' (Stringende)
        if (CanSatMode == 1) senddata();                           //   Payload senden bei Modus 1 (Bild senden)
        if (CanSatMode == 2) {                                     //   Bei Modus 2...
          Serial1.println(payload);                                //   ...Payload nur seriell an den Datalogger schicken bei Modus 2 (Bild nur speichern), nicht senden!
          if (debug) Serial.println(payload);
        }
        

        clearPayload();                                            //      Payload leeren
        ploadPos = 0;                                              //      Position zurücksetzen
        
        intervall ();                                     // ----------- INTERVALLPRÜFUNG ! ----------- (darf auch während der Bildverarbeitung nicht unterbrochen werden!)

      }
      
    }
    jpglen -= bytesToRead;                                         // von der verbleibenden Bildgröße die Paketgröße abziehen, bevor Schleife erneut ausgeführt wird.
  }
  if(oldcount!=0) {                                                // Wenn Schleife abgelaufen ist, sind noch die restlichen Bytes (<=3) übrig.
     binToAscii(base64Text, oldest, older, old, oldcount);         //  ...Werte an Base64-Encoder übergeben...
     for (k=0; k <= 3; k++) {                                      //  ...und Ergebnis in die Payload eintragen
       payload[ploadPos] = base64Text[k];
       ploadPos++;
     }
  }
  for (k = 0; k < sizeof(htmlOut) - 1; k++){                       //   ...den html-String der Payload hinzufügen
    payload[ploadPos] = htmlOut[k];
    ploadPos++;
  }
  payload[ploadPos] = 0;                                           //     letztes Payloadzeichen ist eine '0' (Stringende)
  if (CanSatMode == 1) senddata();                                 //   Payload senden bei Modus 1 (Bild senden)
  if (CanSatMode == 2) {                                           //   Bei Modus 2...
    Serial1.println(payload);                                      //   ...Payload nur seriell an den Datalogger schicken bei Modus 2 (Bild nur speichern), nicht senden!
    if (debug) Serial.println(payload);
  }
  clearPayload();                                                  //   Payload leeren
  ploadPos = 0;                                                    //   Position zurücksetzen
  
}



//                                                 .------------------------------------.
//                                                 |           INITIALISIERUNG          |
//                                                 '------------------------------------'
void setup()
{
  //                   Initialisiert benötigte serielle Verbindungen für ...
  Serial1.begin(9600);                  // GPS

  cam.setBaud9600();                    // Kamera
  cam.begin();

  if (debug) {
    Serial.begin(9600);                 // Rechner (Debugging)   
    Serial.println("REBOOT");           // Ausgabe um möglicherweise fehlerhaften Neustart anzuzeigen
  }

  //                   Initialisiert BMP-Sensor - Rückgabewert 1=OK / 0=NA
  if (bmp.begin()){
    if (debug) Serial.println("BMP180 init success");
  } else {
    if (debug) Serial.println("BMP180 init fail (disconnected?)\n\n");
  }

  //                   Initialisiert das Funkmodul
  radio.initialize(FREQUENCY,NODEID,NETWORKID);
  radio.writeReg(0x03,0x0D);            // setzt die Baudrate gemäß Datenblatt auf 9600 Baud, siehe auch https://lowpowerlab.com/forum/moteino/rfm69hw-bit-rate-settings/msg1979/#msg1979
  radio.writeReg(0x04,0x05);            // Standard in der Bibliothek ist 55.555 (WTF?!) 1. [viel zu schnell, langsamer -> schmalbandiger -> Besserer Signal-Rausch-Abstand] 2.[55555? Der Entwickler muss betrunken gewesen sein...]
  radio.setHighPower();                 // 'To use the high power capabilities of the RFM69HW'
  radio.encrypt(ENCRYPTKEY);            // AES-Schlüssel
  radio.promiscuous(promiscuousMode);   // ließt alle Pakete im Netzwerk wenn 'true', ließt nur an diesen Node adressierte Pakete wenn 'false'
  if (debug) Serial.println("Transmitting at 433 Mhz");

}

//                                                 .------------------------------------.
//                                                 |            HAUPTSCHLEIFE           |
//                                                 '------------------------------------'
void loop()
{

  if (radio.receiveDone()) receiveCommand();


  // Sendeintervall bei Modus 0
  if ( (CanSatMode == 0) ) {
    intervall();
  }

  // Foto senden bei Modus 1
  if ( CanSatMode == 1 ) {
    Picture();
    CanSatMode = 0;
  }

  if (CanSatMode == 2){
    Picture();
    CanSatMode = 0;
  }

  if (CanSatMode == 3){
      radio.send(GATEWAYID, "pong", 4);
      CanSatMode = 0;
  }

}
