
//    .----------------------------------------------------------------------------------------------------------------------------------------------------------------.
//    |                                                     ComCan - CanSat-Team 2016 des Ökumenischen Gymnasiums                                                      |
//    '----------------------------------------------------------------------------------------------------------------------------------------------------------------'
//     von Louis Fahrenholz, Adrian Lipow, Julien Peteaux, Sandra Vöth (CanSat-Team "ComCon", Bremen) und Daniel Wendt-Fröhlich, DL2AB (Hackerspace Bremen e.V.)
//
//     CC-BY-SA 3.0 - Mai-Sep 2016, Bremen(GER)
//     "Dieses Werk ist unter einer Creative Commons Lizenz vom Typ Namensnennung - Weitergabe unter gleichen Bedingungen 3.0 Deutschland zugänglich. 
//     Um eine Kopie dieser Lizenz einzusehen, konsultieren Sie http://creativecommons.org/licenses/by-sa/3.0/de/ oder wenden Sie sich brieflich an 
//     Creative Commons, Postfach 1866, Mountain View, California, 94042, USA."
//
//     Base64-Code "binToAscii" (Public Domain) von spex@neptune.sinister.com
//
//    ------------------------------------------------------------------------------------------------------------------------------------------------------------------
//        
//     Die Hardware ist das "qbcan v1.1"-Set:
//                                  - Arduino ProMicro *
//                                  - I2C-Temperatur-/Drucksensor-Platine 'BMP180'
//                                  - Funkmodul 'RFM69' *
//                                      * ebenfalls als Module für das Bodenstations-Modem verwendet
//     Zusätzlich werden im CanSat verwendet: 
//                                  - ein generisches GPS-Modul mit NMEA-Ausgabe (RX von Hardware-Serial1, 9600bd), 
//                                  - serielle Kamera 'VC0706' (RX=Pin8 TX=Pin9, 9600bd) 
//                                  - SD-Datalogger (eigener ArduinoNano an TX von Hardware-Serial1, 9600bd).
//
//     Alle Daten der Sensoren werden in ein 50byte großes Paket verpackt, dass gesendet und per Datalogger in einem LogFile gespeichert wird.
//     Das schließt ebenfalls aufgenommene Fotos ein, die ein Base64-Encoding und html/img-Wrapping durchlaufen haben.
//     Es gibt mehrere Betriebsmodi. 
//                 0 - regelmäßiges Senden von GPS-Daten, Temperatur und Druck (default)
//                 1 - Foto 160x120px aufnehmen und senden *
//                 2 - Foto 640x480px aufnehmen und nur speichern *
//                 3 - ein einfacher Ping *
//                     * nach Ablauf wird wieder der Modus 0 eingestellt
//
//    ------------------------------------------------------------------------------------------------------------------------------------------------------------------
//
//                                                                        Bodenstation  |||-[_]
//
//    .----------------------------------------------------------------------------------------------------------------------------------------------------------------.
//    |                                                          UMGEBUNGSPARAMETER UND FUNKTIONEN                                                                     |
//    '----------------------------------------------------------------------------------------------------------------------------------------------------------------'
//                                                 .------------------------------------.
//                                                 |            BIBLIOTHEKEN            |
//                                                 '------------------------------------'
#include <qbcan.h>
#include <Wire.h>
#include <SPI.h>

char command[4];                                 // 3 Zeichen + die obligatorische 0 am Ende
unsigned long pingstart;                         // Zwischenspeicher für Pingzeiten

//                                                 .------------------------------------.
//                                                 |             FUNKMODUL              |
//                                                 '------------------------------------'
//                                Empfängt die Payload mit unten angegebenen Parametern und sendet die Befehle zum CanSat.
//                                RFM69-Bibliothek ist von qbcan / Felix Rusu, LowPowerLab.com, License GPL 3.0. 
//                                Center-Frequenz ist 433MHz, Betriebsart ist FSK mit 2x50kHz Shift.
//                                Die Daten werden in adressierte, AX25-ähnliche Pakete inkl. CRC-Fehlerkorrektur verpackt, siehe auch http://lowpowerlab.com/blog/2013/06/20/rfm69-library/
//                                Die Verschlüsselung ist AES. Sowohl die Paketverwaltung wie auch die Verschlüsselung sind im RFM69-Modul realisiert.
//                                

// TODO: Bitrate heruntersetzen, um Signalrauschabstand zu erhöhen. Derzeit sind 55kBs(WTF?!) in Library eingestellt, 4800 (Modul-Default) reichen jedoch völlig aus.
// https://lowpowerlab.com/forum/moteino/rfm69hw-driverange-test/msg288/?PHPSESSID=a0d72b66bd05470a39ca53a6ad29d940
// https://lowpowerlab.com/forum/moteino/rfm69hw-bit-rate-settings/msg1979/#msg1979

#define NODEID        1                          // Node-ID ist einzigartig für jedes Gerät
#define NETWORKID     166                        // Netzwerk-ID muss bei allen gleich sein
#define GATEWAYID     2                          // Angabe des Empfänger-Nodes
#define ENCRYPTKEY    "OeGymTComConHSHB"         // Verschlüsselungspasswort muss exakt 16 Zeichen lang sein  
RFM69 radio;                                     // Objekt des Transceiver-Moduls
bool promiscuousMode = false;                    // Wenn der promiscuousMode eingeschaltet ist, werden alle Pakete aller Nodes im Netzwerk empfangen

//                   Funktion zum Senden der Kommandos
void sendCommand()                               
{
  if (command[0] == '3') pingstart = millis();   // Wenn ein Ping (=3) gesendet wird, soll der aktuelle Timerwert im Arduino zwischengespeichert werden
  radio.send(GATEWAYID, command, 5);             // sendet den Befehl
  if (command[0] == '3') {                       // schreibt beim Ping-Befehl ein [ping] in die serielle Ausgabe...
    Serial.print("[ping] ");
  }else{                                         // ...ansonsten die Info über den gesendeten Befehl
    Serial.print("[ Befehl gesendet:");
    Serial.print(command);
    Serial.println(" - (0 = Datenintervall / 1 = Foto senden / 2 = Foto speichern / 3 = Ping) ]");
  }

}

//                                                 .------------------------------------.
//                                                 |           INITIALISIERUNG          |
//                                                 '------------------------------------'
void setup()
{
  //                   Initialisiert die serielle Verbindung
  Serial.begin(9600);                            // Rechner (Debugging)   
  Serial.println("REBOOT");                      // Ausgabe um möglicherweise fehlerhaften Neustart anzuzeigen

  //                   Initialisiert das Funkmodul
  radio.initialize(FREQUENCY,NODEID,NETWORKID);
  radio.writeReg(0x03,0x0D);            // setzt die Baudrate gemäß Datenblatt auf 9600 Baud, siehe auch https://lowpowerlab.com/forum/moteino/rfm69hw-bit-rate-settings/msg1979/#msg1979
  radio.writeReg(0x04,0x05);            // Standard in der Bibliothek ist 55.555 (WTF?!) 1. [viel zu schnell, langsamer -> schmalbandiger -> Besserer Signal-Rausch-Abstand] 2.[55555? Der Entwickler muss betrunken gewesen sein...]

  radio.setHighPower();                 // 'To use the high power capabilities of the RFM69HW'
  radio.encrypt(ENCRYPTKEY);            // AES-Schlüssel
  radio.promiscuous(promiscuousMode);   // ließt alle Pakete im Netzwerk wenn 'true', ließt nur an diesen Node adressierte Pakete wenn 'false'


}

//                                                 .------------------------------------.
//                                                 |            HAUPTSCHLEIFE           |
//                                                 '------------------------------------'
void loop()
{
//                   Funktion zum Empfangen und Ausgeben der Payload und des Pongs
  if (radio.receiveDone())
  { 
    for (byte i = 0; i < radio.DATALEN; i++)                      // für jedes Byte des eingehenden Pakets...
      Serial.print((char)radio.DATA[i]);                          // ... den entsprechenden Char seriell ausgeben
    if ( (radio.DATA[1] == ';') && (radio.DATA[40] == ';') ){     // Ausdruck der Feldstärke nur wenn es ein String des Sensorintervalls ist (hier erkannt durch ';' an zwei festen Stellen)
      Serial.print("; [ RX_RSSI ");Serial.print(radio.RSSI);Serial.print(" dBm ]");
    }
    if ( (radio.DATA[0] == 'p') && (radio.DATA[1] == 'o') && (radio.DATA[2] == 'n') && (radio.DATA[3] == 'g') ) { // wenn die ersten 4 Bytes des eingehenden Pakets "pong" sind ...
      unsigned long pongend = millis() - pingstart;               //... die Pingzeit errechnen
      Serial.print(" [in "); Serial.print(pongend); Serial.print(" ms]");  // und ausgeben
    }
    Serial.println();
  }
  
//                   Funktion zum Eingeben und Senden der Befehle
  if (Serial.available()) {                                       // Wird nur ausgeführt, wenn auch eingehende Bytes erkannt wurden
    char inChar = (char)Serial.read();                            // speichert die eingegangene Eingabe zwischen
    if ( (inChar == '0') || (inChar == '1') || (inChar == '2') || (inChar == '3') ) {   // "Filtert" alles, was nicht 0-3 ist
      for (byte i = 0; i <= 2; i++) {                             // setzt valide Befehle 3mal hintereinander in einen 3-Zeichen Kommando-String
        command[i]=inChar;
      }
      command[3]=0;                                               // die obligatorische 0 am Ende jedes Strings
      sendCommand();                                              // sendet das Kommando
    }else{                                                        // sollte etwas anderes als ein Zeichen von 0 bis 3 eingegangen sein, Fehlermeldung ausgeben.
      Serial.println("folgende Befehle sind möglich: 0 = Datenintervall / 1 = Foto senden / 2 = Foto speichern / 3 = Ping");
    }
  }

  
}



